'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

exports.buildConfig = buildConfig;
exports.install = install;

var _knex = require('knex');

var _knex2 = _interopRequireDefault(_knex);

var _override = require('./override');

var _debug = require('./debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('client-multi-tenant');

/**
 * Build knex tenant configuration changing details related to multitenant
 */
function buildConfig(config, tenantId) {
  var multitenantConfig = (0, _extends3.default)({}, config || {}, {
    tenantId: tenantId
  });

  multitenantConfig.migrations = multitenantConfig.migrations || {};

  // custom migration with the table name prefix
  multitenantConfig.migrations.tableName = tenantId + '_' + (multitenantConfig.migrations.tableName || 'knex_migrations');

  return multitenantConfig;
}

/**
 * Installs the tenant monkey patch on knex.
 *
 * It overrides some base knex functions changing the original behavior to
 * include multitenant configurations such as the tenant prefix in every table.
 */
function install() {
  Object.defineProperty(_knex2.default.Client.prototype, 'tenantId', {
    get: function get() {
      return this.config.tenantId;
    }
  });

  (0, _override.override)(_knex2.default.Client.prototype.QueryBuilder.prototype, 'toSQL', (0, _override.after)(function (sql) {
    debug('knex.Client.prototype.QueryBuilder.prototype.toSQL', arguments);
    sql.sql = applyTenant(sql.sql, this.client.tenantId);
    return sql;
  }));

  (0, _override.override)(_knex2.default.Client.prototype.Raw.prototype, 'set', (0, _override.before)(function (sql, bindings) {
    debug('knex.Client.prototype.Raw.prototype.set', arguments);
    var tenantSQL = applyTenant(sql, this.client.tenantId);
    return [tenantSQL, bindings];
  }));

  (0, _override.override)(_knex2.default.Client.prototype.SchemaBuilder.prototype, 'toSQL', (0, _override.after)(function (sql) {
    var _this = this;

    debug('knex.Client.prototype.SchemaBuilder.prototype.toSQL', arguments);
    return sql.map(function (q) {
      q.sql = applyTenant(q.sql, _this.client.tenantId);
      return q;
    });
  }));

  (0, _override.override)(_knex2.default.Client.prototype.Runner.prototype, 'query', (0, _override.after)(function () {
    var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(promise, originalArgs) {
      var _this2 = this;

      var options,
          result,
          _args = arguments;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              debug('knex.Client.prototype.Runner.prototype.query', _args);
              options = originalArgs[0].options;
              _context.next = 4;
              return promise;

            case 4:
              result = _context.sent;

              if (!(!options || !options.nestTables)) {
                _context.next = 7;
                break;
              }

              return _context.abrupt('return', result);

            case 7:
              return _context.abrupt('return', result.map(function (row) {
                return (0, _keys2.default)(row).reduce(function (processedRow, tableJoinName) {
                  processedRow[unapplyTenant(tableJoinName, _this2.client.tenantId)] = row[tableJoinName];
                  return processedRow;
                }, {});
              }));

            case 8:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this);
    }));
    return function (_x, _x2) {
      return ref.apply(this, arguments);
    };
  }()));
}

function unapplyTenant(sql, tenant) {
  var regexp = new RegExp('^(' + tenant + '_)');
  return sql.replace(regexp, '$_');
}

function applyTenant(sql, tenant) {
  return sql.replace(/\$_/g, tenant + '_');
}