'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _knex = require('knex');

var _knex2 = _interopRequireDefault(_knex);

var _promiseAsyncCache = require('promise-async-cache');

var _promiseAsyncCache2 = _interopRequireDefault(_promiseAsyncCache);

var _knexTenantSupport = require('./knex-tenant-support');

var knexTenantSupport = _interopRequireWildcard(_knexTenantSupport);

var _debug = require('./debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var debug = (0, _debug2.default)('tenant');

/**
 * Defines whenever the tenant monkey patch on knex has been installed
 */
var isMultiTenantSupportInstalled = false;

/**
 * Keep knex tenants in memory and handle race conditions
 */
var cache = new _promiseAsyncCache2.default({
  /**
   * Build knex tenant and load it to the cache
   */

  load: function load(tenantId, baseKnex) {
    var _this = this;

    return (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      var proxyKnex;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (isMultiTenantSupportInstalled) {
                _context.next = 11;
                break;
              }

              _context.prev = 1;

              debug('installing knextancy');
              knexTenantSupport.install();
              isMultiTenantSupportInstalled = true;
              _context.next = 11;
              break;

            case 7:
              _context.prev = 7;
              _context.t0 = _context['catch'](1);

              debug('Error installing knex multi tenant support', _context.t0.stack || _context.t0);
              throw _context.t0;

            case 11:

              debug('building knex for new tenant %d', tenantId);
              proxyKnex = (0, _knex2.default)(knexTenantSupport.buildConfig(baseKnex.client.config, tenantId));


              Object.defineProperty(proxyKnex, 'tenantId', {
                get: function get() {
                  return this.client.tenantId;
                }
              });

              debug('initializing multi tenant database');
              debug('running migration tasks');
              _context.next = 18;
              return proxyKnex.migrate.latest();

            case 18:

              debug('running seed tasks');
              _context.next = 21;
              return proxyKnex.seed.run();

            case 21:
              return _context.abrupt('return', proxyKnex);

            case 22:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this, [[1, 7]]);
    }))();
  }
});

exports.default = function () {
  var ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(baseKnex, tenantId) {
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.prev = 0;
            return _context2.abrupt('return', cache.get(tenantId, baseKnex));

          case 4:
            _context2.prev = 4;
            _context2.t0 = _context2['catch'](0);

            debug('Error on initializing the multi tenant database', _context2.t0.stack || _context2.t0);
            throw _context2.t0;

          case 8:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this, [[0, 4]]);
  }));
  return function (_x, _x2) {
    return ref.apply(this, arguments);
  };
}();